Package: carts
Type: Package
Title: Simulation-Based Assessment of Covariate Adjustment in
        Randomized Trials
Version: 0.1.0
Authors@R: c(person(given = "Benedikt",
                    family = "Sommer",
                    role = c("aut", "cre"),
                    email = "benediktsommer92@gmail.com"),
            person(given = "Klaus K.",
                    family = "Holst",
                    role = c("aut"),
                    email = "klaus@holst.it"),
            person(given = "Foroogh",
                    family = "Shamsi",
                    role = c("aut"),
                    email = "foroogh.shamsi1@gmail.com"),
            person("Novo Nordisk A/S", role = "cph")
            )
Maintainer: Benedikt Sommer <benediktsommer92@gmail.com>
Description: Monte Carlo simulation framework for different randomized clinical
    trial designs with a special emphasis on estimators based on covariate
    adjustment. The package implements regression-based covariate adjustment
    (Rosenblum & van der Laan (2010) <doi:10.2202/1557-4679.1138>) and a
    one-step estimator (Van Lancker et al (2024)
    <doi:10.48550/arXiv.2404.11150>) for trials with continuous, binary and
    count outcomes. The estimation of the minimum sample-size required to reach
    a specified statistical power for a given estimator uses bisection to find
    an initial rough estimate, followed by stochastic approximation
    (Robbins-Monro (1951) <doi:10.1214/aoms/1177729586>) to improve the
    estimate, and finally, a grid search to refine the estimate in the
    neighborhood of the current best solution.
License: Apache License (>= 2)
Depends: R (>= 4.1), lava (>= 1.8.0)
Imports: data.table (>= 1.14), logger (>= 0.2.2), methods, progressr,
        R6 (>= 2.5.1), survival, targeted (>= 0.6), rlang
LinkingTo: Rcpp (>= 1.0.11), RcppArmadillo
Suggests: future, tinytest, MASS, geepack, covr, pdftools, knitr, mets
        (>= 1.3.4), rmarkdown, magick, pwr, pwrss
BugReports: https://github.com/NovoNordisk-OpenSource/carts/issues
URL: https://novonordisk-opensource.github.io/carts/,
        https://github.com/NovoNordisk-OpenSource/carts
VignetteBuilder: knitr
ByteCompile: yes
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2025-11-11 09:09:54 UTC; bkts
Author: Benedikt Sommer [aut, cre],
  Klaus K. Holst [aut],
  Foroogh Shamsi [aut],
  Novo Nordisk A/S [cph]
Repository: CRAN
Date/Publication: 2025-11-13 21:20:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 16:57:34 UTC; unix
Archs: carts.so.dSYM
