# Generated by roxygen2: do not edit by hand

S3method("append<-",list)
S3method(coef,samplesize_estimate)
S3method(print,samplesize_estimate)
S3method(print,trial.estimates)
export("%join%")
export("append<-")
export(Trial)
export(aggrsurv)
export(covar_add)
export(covar_bootstrap)
export(covar_join)
export(covar_loggamma)
export(covar_normal)
export(derive_covar_distribution)
export(est_adj)
export(est_gee)
export(est_geebin)
export(est_glm)
export(est_glmbin)
export(estimate_covar_model_full_cond)
export(outcome_binary)
export(outcome_continuous)
export(outcome_count)
export(rmvn)
export(rnb)
export(sample_covar_parametric_model)
export(setallargs)
export(setargs)
import(data.table)
importFrom(R6,R6Class)
importFrom(lava,distribution)
importFrom(lava,estimate)
importFrom(lava,lvm)
importFrom(lava,regression)
importFrom(lava,sim)
importFrom(logger,log_error)
importFrom(logger,log_info)
importFrom(logger,log_warn)
importFrom(methods,formalArgs)
importFrom(rlang,abort)
importFrom(rlang,call_match)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,delete.response)
importFrom(stats,formula)
importFrom(stats,gaussian)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.offset)
importFrom(stats,na.omit)
importFrom(stats,nlminb)
importFrom(stats,pnbinom)
importFrom(stats,poisson)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,rbinom)
importFrom(stats,rgamma)
importFrom(stats,rnbinom)
importFrom(stats,rnorm)
importFrom(stats,rpois)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,terms)
importFrom(stats,time)
importFrom(stats,update)
importFrom(survival,Surv)
importFrom(survival,strata)
importFrom(survival,survSplit)
importFrom(targeted,cv)
importFrom(targeted,learner)
importFrom(targeted,learner_glm)
importFrom(utils,capture.output)
importFrom(utils,head)
importFrom(utils,str)
importFrom(utils,tail)
useDynLib(carts, .registration=TRUE)
